
package client;

/**
 *
 * Created on : 15.01.2011
 * @author Sheroz Khaydarov
 */
import common.CategoryList;
import common.ProductList;

import org.apache.commons.httpclient.HttpClient;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.servlet.ModelAndView;

import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.client.RestTemplate;
import org.apache.commons.httpclient.UsernamePasswordCredentials;
import org.apache.commons.httpclient.auth.AuthScope;
import org.springframework.http.client.CommonsClientHttpRequestFactory;
import org.springframework.web.bind.annotation.RequestHeader;
import org.springframework.web.bind.annotation.RequestParam;

@Controller
@RequestMapping("/client")
public class ClientController {
        private RestTemplate restTemplate;
        private UsernamePasswordCredentials credentials;

        private long categoryId;
        private String sortOrder;

        public void setRestTemplate(RestTemplate restTemplate) {
            this.restTemplate = restTemplate;
        }

        public void setCredentials(UsernamePasswordCredentials credentials) {
            this.credentials = credentials;
        }
        

	private static HttpEntity<String> prepareGet(MediaType type) {
		HttpHeaders headers = new HttpHeaders();
		headers.setContentType(type);
		HttpEntity<String> entity = new HttpEntity<String>(headers);
		return entity;
	}

	@RequestMapping(method=RequestMethod.GET)
        public ModelAndView renderPage(@RequestParam(value="selectCategory", required=false) String paramCategoryId, @RequestParam(value="sortOrder", required=false) String paramSortOrder, @RequestHeader("Host") String host)
        {
                ModelAndView modelAndView = new ModelAndView("eshop");
                Boolean authError = false;
                String urlBase="http://" + host + "/eshop";

                try {

                    CommonsClientHttpRequestFactory factory = (CommonsClientHttpRequestFactory) restTemplate.getRequestFactory();
                    HttpClient client = factory.getHttpClient();
                    client.getState().setCredentials(AuthScope.ANY, credentials);
                    
                    HttpEntity<String> entity = prepareGet(MediaType.APPLICATION_XML);

                    ResponseEntity<CategoryList> categoryListResponse = restTemplate.exchange(
                                    urlBase+"/service/category",
                                    HttpMethod.GET, entity, CategoryList.class);

                    CategoryList categoryList = categoryListResponse.getBody();
                    modelAndView.addObject("categoryList", categoryList);

                    categoryId = -1;
                    if (paramCategoryId==null)
                        paramCategoryId="";

                    sortOrder = paramSortOrder;
                    if (sortOrder==null)
                        sortOrder="";
                    if (sortOrder.isEmpty())
                        sortOrder= "asc";

                    ProductList productList = null;
                    if (categoryList!=null && categoryList.getCategories()!=null && categoryList.getCategories().size()>0)
                    {

                        if (paramCategoryId.isEmpty())
                            this.categoryId = categoryList.getCategories().get(0).getId();
                        else    
                            this.categoryId = Long.parseLong(paramCategoryId);

                        ResponseEntity<ProductList> productListResponse = restTemplate.exchange(
                                        urlBase+"/service/category/"+categoryId+"/product?sortOrder="+sortOrder,
                                        HttpMethod.GET, entity, ProductList.class);

                        productList = productListResponse.getBody();
                    }
                    modelAndView.addObject("productList", productList);

                } catch (Exception e){
                    authError = true;
                }

                modelAndView.addObject("authError", authError);
                modelAndView.addObject("categoryId", categoryId);
                modelAndView.addObject("sortOrder", sortOrder);

		return modelAndView;
        }
}
